# Repository path:    $HeadURL: https://svn.dmhoutribdijk.nl/DMS-Houtribdijk/code/python-ftp-svn/source/MonitorStatus.py $
# Last committed:     $Revision: 130491 $; $Author: mvhoek $; $Date: 2019-05-29 12:18:27 +0200 (Wed, 29 May 2019) $
#
from misc.Utils import Utils
from dmsLogger.DmsLogger import DmsLogger
import pandas as pd
import json
import sys
import getopt
import glob
import os
import re


# Function to read _STA files from ftp and check if data is available by reading STA files from ftp (from Mattijn)
def timestamp_status_data(settings):

    list_of_status_files = glob.glob(
        os.path.join(settings["folder_ftpstatus_status"], "*STA_FL*.json")
    )

    df_status_data_sensor_loc_ts = pd.DataFrame()
    for status_file in list_of_status_files:
        key_status_loc = Utils.get_location_from_filename(os.path.basename(status_file), settings["location_pattern"])
        if key_status_loc == "":
            continue

        # find the most recent file in the data folder
        # do not use the file listed in the ftpStatusFile: this may be an _g_ file, which we are not interested in
        dataFolder = os.path.join(settings["folder_ftpstatus_data"], key_status_loc.replace("_", "\\"))
        list_of_files = glob.glob(dataFolder + "/STA_FL*")
        path_status_data_file = max(list_of_files, key=os.path.getctime)

        # get path of status data file
        df_status_status = pd.read_json(status_file, typ="series", orient="values")
        status_file_ts = pd.to_datetime(df_status_status["ts"])

        # read status data file
        df_status_data = pd.read_csv(path_status_data_file, skiprows=[0, 2, 3])

        sensors = [
            "ETRO_Status",
            "ISA_Status",
            "YSI_Status",
            "AQD_Status",
            "VEC_Status",
            "RDI_Status",
        ]
        s_status_sensors = df_status_data[sensors].isin([-1]).any()
        s_status_sensors.index = [sensor.split("_")[0] for sensor in sensors]

        df_status_sensors = s_status_sensors.to_frame()
        df_status_sensors.reset_index(inplace=True)
        df_status_sensors.columns = ["data_sensor", "data_available"]

        df_status_sensors.loc[:, "file_ftpstatus"] = status_file
        df_status_sensors.loc[:, "key_ftpstatus"] = key_status_loc
        df_status_sensors.loc[:, "ts_ftpstatus"] = status_file_ts

        df_status_data_sensor_loc_ts = df_status_data_sensor_loc_ts.append(
            df_status_sensors
        )

    return df_status_data_sensor_loc_ts


# Rename certain parameters in dataframe


def rename_df(df_status_data_sensor_loc_ts):

    # Rename the instruments
    instr_sta = ["ETRO", "ISA", "YSI", "AQD", "VEC", "RDI"]
    instr_ftp = ["STB", "ALT", "MPP", "AQD", "ADV", "ADC"]
    dict_instr = dict(zip(instr_sta, instr_ftp))
    df_status_data_sensor_loc_ts["data_sensor"] = df_status_data_sensor_loc_ts[
        "data_sensor"
    ].replace(dict_instr, regex=True)

    # Rename the locations
    loc_sta = ["FL65_A", "FL65_B", "FL65_C", "FL69_A", "FL69_B", "FL69_C"]
    loc_ftp = ["FL65A", "FL65B", "FL65C", "FL69A", "FL69B", "FL69C"]
    dict_loc = dict(zip(loc_sta, loc_ftp))
    df_status_data_sensor_loc_ts["key_ftpstatus"] = df_status_data_sensor_loc_ts[
        "key_ftpstatus"
    ].replace(dict_loc, regex=True)

    # Rename the columns (easier to merge dataframes later)
    df_status_data_sensor_loc_ts.rename(
        columns={"key_ftpstatus": "location"}, inplace=True
    )

    #df_status = df_status_data_sensor_loc_ts.copy()

    return df_status_data_sensor_loc_ts


# Dataframe of the timestamps of the _INSTR files from the ftp


def timestamp_status_instrument(folder_ftpstatus_status):
    ftp_status = []
    for file in glob.glob(folder_ftpstatus_status + "/*FL*"):
        if file.endswith(".json"):
            # read json file where fn = filename and ts = timestamp
            ftp = pd.read_json(
                os.path.join(folder_ftpstatus_status, file),
                typ="series",
                orient="values",
            )
            m = re.search("(FL.*?)_.*?", ftp["fn"])
            pair = {
                "data_sensor": ftp["fn"][0:3],
                "location": m.group(1),
                "ts": pd.to_datetime(ftp["ts"]),
            }
            ftp_status.append(pair)

    # Create dataframe from dictionary with instrument, location, timestamp
    df_ftp = pd.DataFrame.from_dict(ftp_status)

    return df_ftp


# Merge dataframes and compute timedelta of _STA and _INSTR

# column ts_ftpstatus originates from the timestamp of the status file of the STA file
# column ts originates from the timestamp of the status file of the instrument & location combination


def compute_timedelta(df_status, df_ftp):

    # Merge dataframes
    #df = df_status.merge(right=df_ftp, on=["data_sensor", "location"])
    df = df_status.merge(right=df_ftp, left_on=["data_sensor", "location"], right_on=["data_sensor", "location"])

    # Compute timedelta between status import and data import
    df["ts_delta"] = df["ts_ftpstatus"] - df["ts"]

    return df


# Json file with error message if at locations where instrument is active, the timestamp of data import is not more than 2 hours behind timestamp of status import


def error_message(df):

    # Check if timedelta is more than 2 hours and data is available
    msg_filter = [
        all(tup)
        for tup in zip(df.data_available == True, df.ts_delta > pd.Timedelta(hours=2))
    ]
    df_msg = df  # [msg_filter]

    # add columns for prtg
    df_msg.loc[:, "channel"] = df_msg["location"] + " - " + df_msg["data_sensor"]
    df_msg.loc[:, "value_error"] = 0
    df_msg.loc[msg_filter, "value_error"] = 2
    df_msg.loc[:, "limitmaxerror"] = 1
    df_msg.loc[:, "limitmode"] = 1
    df_msg.loc[:, "limiterrormsg"] = "Actieve sensor levert data op de FTP"
    df_msg.loc[msg_filter, "limiterrormsg"] = (
        "Melding: Aanlevering data op ftp van "
        + df_msg["data_sensor"]
        + " op locatie "
        + df_msg["location"]
        + " is zorgelijk rond "
        + df_msg.ts_ftpstatus.dt.strftime("%Y-%m-%d %H:%M")
    )

    # filter columns to output
    df_msg_out = df_msg[
        ["channel", "value_error", "limitmaxerror", "limitmode", "limiterrormsg"]
    ]
    df_msg_out.columns = [
        "channel",
        "value",
        "limitmaxerror",
        "limitmode",
        "limiterrormsg",
    ]

    return df_msg_out


def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("MonitorDataWhereStatus.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("MonitorDataWhereStatus.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        settings = Utils.read_monitor_data_where_status_settings(settings_file)
    except FileNotFoundError:
        exit(1)

    logger = DmsLogger.set_logger(
        settings["log_folder"], settings["logger"], settings["logfile"]
    )
    logger.info("Run_MonitorDataWhereStatus")
    logger.info(settings["folder_ftpstatus_status"])

    df_status = rename_df(timestamp_status_data(settings))
    df_ftp = timestamp_status_instrument(settings["folder_ftpstatus_status"])
    df = compute_timedelta(df_status, df_ftp)
    df_msg_out = error_message(df)

    # save to file
    error_out = {"prtg": {"result": df_msg_out.to_dict(orient="records")}}
    with open(settings["file_monitor_out"], "w") as f:
        print(json.dumps(error_out), file=f)


if __name__ == "__main__":
    main(sys.argv[1:])
    # fire.Fire()
